(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.injectStore = factory());
}(this, (function () { 'use strict';

  const DEFAULT_STORE_NAME = 'config';
  const EVENT_INIT = 'vuex:init';
  const EVENT_SYNC = 'vuex:sync';

  function getEventConst (storeName = DEFAULT_STORE_NAME) {
    return {
      EVENT_INIT: `${storeName}:${EVENT_INIT}`,
      EVENT_SYNC: `${storeName}:${EVENT_SYNC}`
    }
  }

  /** @type {import('types/background').default} */
  function background (ipcMain, store, storeName = DEFAULT_STORE_NAME) {
    if (!ipcMain || !store) {
      throw new Error('ipcMain or store is required!')
    }

    const senderSet = new Set(); // 存储所有需要同步的窗口的set
    const EVENT = getEventConst(storeName); // 事件

    /**
     * 本地store改变需要通知
     * @param {*} state
     * @param {*} ignoreSender
     */
    const storeChangeHandler = function (state, ignoreSender) {
      senderSet.forEach((sender) => {
        if (!ignoreSender || sender.id !== ignoreSender.id) {
          if (sender.isDestroyed()) {
            senderSet.delete(sender);
          } else {
            sender.send(EVENT.EVENT_SYNC, state);
          }
        }
      });
    };

    const subscribeStoreChange = function () {
      return store.onDidAnyChange((state) => {
        storeChangeHandler(state);
      })
    };

    // 解除监听用的方法
    let unsubscribeStoreChange = subscribeStoreChange();

    /**
     * 同步新加入的窗口
     */
    ipcMain.on(EVENT.EVENT_INIT, ({ sender }) => {
      senderSet.add(sender);
      sender.send(EVENT.EVENT_SYNC, store.store);
    });

    /**
     * 自动同步客户端到store
     */
    ipcMain.on(EVENT.EVENT_SYNC, ({ sender }, state) => {
      /**
       * 如果是从页面开始进行同步的化，需要先让store停止监听，整体修改之后再进行监听
       */
      if (unsubscribeStoreChange) {
        unsubscribeStoreChange();
        unsubscribeStoreChange = null;
        store.store = state;
        storeChangeHandler(state, sender);
        unsubscribeStoreChange = subscribeStoreChange();
      }
    });

    return store
  }

  return background;

})));
